@@ -4,17 +4,30 @@
 
 "use strict";
 
-const Services = require("Services");
-const promise = require("promise");
-const Rule = require("devtools/client/inspector/rules/models/rule");
-const UserProperties = require("devtools/client/inspector/rules/models/user-properties");
-const { ELEMENT_STYLE } = require("devtools/shared/specs/styles");
+const Rule = require("resource://devtools/client/inspector/rules/models/rule.js");
+const UserProperties = require("resource://devtools/client/inspector/rules/models/user-properties.js");
+const {
+  style: { ELEMENT_STYLE },
+} = require("resource://devtools/shared/constants.js");
 
-loader.lazyRequireGetter(this, "promiseWarn", "devtools/client/inspector/shared/utils", true);
-loader.lazyRequireGetter(this, "parseDeclarations", "devtools/shared/css/parsing-utils", true);
-loader.lazyRequireGetter(this, "parseNamedDeclarations", "devtools/shared/css/parsing-utils", true);
-loader.lazyRequireGetter(this, "parseSingleValue", "devtools/shared/css/parsing-utils", true);
-loader.lazyRequireGetter(this, "isCssVariable", "devtools/shared/fronts/css-properties", true);
+loader.lazyRequireGetter(
+  this,
+  "promiseWarn",
+  "resource://devtools/client/inspector/shared/utils.js",
+  true
+);
+loader.lazyRequireGetter(
+  this,
+  ["parseDeclarations", "parseNamedDeclarations", "parseSingleValue"],
+  "resource://devtools/shared/css/parsing-utils.js",
+  true
+);
+loader.lazyRequireGetter(
+  this,
+  "isCssVariable",
+  "resource://devtools/client/fronts/css-properties.js",
+  true
+);
 
 const PREF_INACTIVE_CSS_ENABLED = "devtools.inspector.inactive.css.enabled";
 
@@ -44,10 +57,11 @@ class ElementStyle {
     this.ruleView = ruleView;
     this.store = store || {};
     this.pageStyle = pageStyle;
+    this.pseudoElements = [];
     this.showUserAgentStyles = showUserAgentStyles;
     this.rules = [];
     this.cssProperties = this.ruleView.cssProperties;
-    this.variables = new Map();
+    this.variablesMap = new Map();
 
     // We don't want to overwrite this.store.userProperties so we only create it
     // if it doesn't already exist.
@@ -58,20 +72,14 @@ class ElementStyle {
     if (!("disabled" in this.store)) {
       this.store.disabled = new WeakMap();
     }
-
-    this.onRefresh = this.onRefresh.bind(this);
-
-    if (this.ruleView.isNewRulesView) {
-      this.pageStyle.on("stylesheet-updated", this.onRefresh);
-      this.ruleView.inspector.styleChangeTracker.on("style-changed", this.onRefresh);
-      this.ruleView.selection.on("pseudoclass", this.onRefresh);
-    }
   }
 
   get unusedCssEnabled() {
     if (!this._unusedCssEnabled) {
-      this._unusedCssEnabled =
-        Services.prefs.getBoolPref(PREF_INACTIVE_CSS_ENABLED, false);
+      this._unusedCssEnabled = Services.prefs.getBoolPref(
+        PREF_INACTIVE_CSS_ENABLED,
+        false
+      );
     }
     return this._unusedCssEnabled;
   }
@@ -82,6 +90,7 @@ class ElementStyle {
     }
 
     this.destroyed = true;
+    this.pseudoElements = [];
 
     for (const rule of this.rules) {
       if (rule.editor) {
@@ -90,12 +99,6 @@ class ElementStyle {
 
       rule.destroy();
     }
-
-    if (this.ruleView.isNewRulesView) {
-      this.pageStyle.off("stylesheet-updated", this.onRefresh);
-      this.ruleView.inspector.styleChangeTracker.off("style-changed", this.onRefresh);
-      this.ruleView.selection.off("pseudoclass", this.onRefresh);
-    }
   }
 
   /**
@@ -116,52 +119,56 @@ class ElementStyle {
    * ready.
    */
   populate() {
-    const populated = this.pageStyle.getApplied(this.element, {
-      inherited: true,
-      matchedSelectors: true,
-      filter: this.showUserAgentStyles ? "ua" : undefined,
-    }).then(entries => {
-      if (this.destroyed || this.populated !== populated) {
-        return promise.resolve(undefined);
-      }
-
-      // Store the current list of rules (if any) during the population
-      // process. They will be reused if possible.
-      const existingRules = this.rules;
-
-      this.rules = [];
-
-      for (const entry of entries) {
-        this._maybeAddRule(entry, existingRules);
-      }
-
-      // Mark overridden computed styles.
-      this.onRuleUpdated();
-
-      this._sortRulesForPseudoElement();
-
-      if (this.ruleView.isNewRulesView) {
-        this.subscribeRulesToLocationChange();
-      }
-
-      // We're done with the previous list of rules.
-      for (const r of existingRules) {
-        if (r && r.editor) {
-          r.editor.destroy();
+    const populated = this.pageStyle
+      .getApplied(this.element, {
+        inherited: true,
+        matchedSelectors: true,
+        filter: this.showUserAgentStyles ? "ua" : undefined,
+      })
+      .then(entries => {
+        if (this.destroyed || this.populated !== populated) {
+          return Promise.resolve(undefined);
         }
 
-        r.destroy();
-      }
+        // Store the current list of rules (if any) during the population
+        // process. They will be reused if possible.
+        const existingRules = this.rules;
 
-      return undefined;
-    }).catch(e => {
-      // populate is often called after a setTimeout,
-      // the connection may already be closed.
-      if (this.destroyed) {
-        return promise.resolve(undefined);
-      }
-      return promiseWarn(e);
-    });
+        this.rules = [];
+
+        for (const entry of entries) {
+          this._maybeAddRule(entry, existingRules);
+        }
+
+        // Store a list of all pseudo-element types found in the matching rules.
+        this.pseudoElements = this.rules
+          .filter(r => r.pseudoElement)
+          .map(r => r.pseudoElement);
+
+        // Mark overridden computed styles.
+        this.onRuleUpdated();
+
+        this._sortRulesForPseudoElement();
+
+        // We're done with the previous list of rules.
+        for (const r of existingRules) {
+          if (r?.editor) {
+            r.editor.destroy();
+          }
+
+          r.destroy();
+        }
+
+        return undefined;
+      })
+      .catch(e => {
+        // populate is often called after a setTimeout,
+        // the connection may already be closed.
+        if (this.destroyed) {
+          return Promise.resolve(undefined);
+        }
+        return promiseWarn(e);
+      });
     this.populated = populated;
     return this.populated;
   }
@@ -169,12 +176,14 @@ class ElementStyle {
   /**
    * Returns the Rule object of the given rule id.
    *
-   * @param  {String} id
+   * @param  {String|null} id
    *         The id of the Rule object.
    * @return {Rule|undefined} of the given rule id or undefined if it cannot be found.
    */
   getRule(id) {
-    return this.rules.find(rule => rule.domRule.actorID === id);
+    return id
+      ? this.rules.find(rule => rule.domRule.actorID === id)
+      : undefined;
   }
 
   /**
@@ -186,9 +195,14 @@ class ElementStyle {
   getUsedFontFamilies() {
     return new Promise((resolve, reject) => {
       this.ruleView.styleWindow.requestIdleCallback(async () => {
+        if (this.element.isDestroyed()) {
+          resolve([]);
+          return;
+        }
         try {
-          const fonts = await this.pageStyle.getUsedFontFaces(
-            this.element, { includePreviews: false });
+          const fonts = await this.pageStyle.getUsedFontFaces(this.element, {
+            includePreviews: false,
+          });
           resolve(fonts.map(font => font.CSSFamilyName));
         } catch (e) {
           reject(e);
@@ -220,8 +234,10 @@ class ElementStyle {
   _maybeAddRule(options, existingRules) {
     // If we've already included this domRule (for example, when a
     // common selector is inherited), ignore it.
-    if (options.system ||
-        (options.rule && this.rules.some(rule => rule.domRule === options.rule))) {
+    if (
+      options.system ||
+      (options.rule && this.rules.some(rule => rule.domRule === options.rule))
+    ) {
       return false;
     }
 
@@ -230,7 +246,7 @@ class ElementStyle {
     // If we're refreshing and the rule previously existed, reuse the
     // Rule object.
     if (existingRules) {
-      const ruleIndex = existingRules.findIndex((r) => r.matches(options));
+      const ruleIndex = existingRules.findIndex(r => r.matches(options));
       if (ruleIndex >= 0) {
         rule = existingRules[ruleIndex];
         rule.refresh(options);
@@ -256,50 +272,42 @@ class ElementStyle {
    * Calls updateDeclarations with all supported pseudo elements
    */
   onRuleUpdated() {
-    this.variables.clear();
     this.updateDeclarations();
 
-    for (const pseudo of this.cssProperties.pseudoElements) {
+    // Update declarations for matching rules for pseudo-elements.
+    for (const pseudo of this.pseudoElements) {
       this.updateDeclarations(pseudo);
     }
   }
 
   /**
-   * Mark the declarations for a given pseudo element with an overridden flag if
-   * an earlier property overrides it and update the editor to show it in the
-   * UI. If there is any inactive CSS we also update the editors state to show
-   * the inactive CSS icon.
+   * Go over all CSS rules matching the selected element and mark the CSS declarations
+   * (aka TextProperty instances) with an `overridden` Boolean flag if an earlier or
+   * higher priority declaration overrides it. Rules are already ordered by specificity.
+   *
+   * If a pseudo-element type is passed (ex: ::before, ::first-line, etc),
+   * restrict the operation only to declarations in rules matching that pseudo-element.
+   *
+   * At the end, update the declaration's view (TextPropertyEditor instance) so it relects
+   * the latest state. Use this opportunity to also trigger checks for the "inactive"
+   * state of the declaration (whether it has effect or not).
    *
    * @param  {String} pseudo
-   *         Which pseudo element to flag as overridden.
-   *         Empty string or undefined will default to no pseudo element.
+   *         Optional pseudo-element for which to restrict marking CSS declarations as
+   *         overridden.
    */
   updateDeclarations(pseudo = "") {
-    // Gather all the text properties applied by these rules, ordered
-    // from more- to less-specific. Text properties from keyframes rule are
-    // excluded from being marked as overridden since a number of criteria such
-    // as time, and animation overlay are required to be check in order to
-    // determine if the property is overridden.
-    const textProps = [];
-    for (const rule of this.rules) {
-      if ((rule.matchedSelectors.length > 0 ||
-           rule.domRule.type === ELEMENT_STYLE) &&
-          rule.pseudoElement === pseudo && !rule.keyframes) {
-        for (const textProp of rule.textProps.slice(0).reverse()) {
-          if (textProp.enabled) {
-            textProps.push(textProp);
-          }
-        }
-      }
-    }
-
-    // Gather all the computed properties applied by those text
-    // properties.
+    // Gather all text properties applicable to the selected element or pseudo-element.
+    const textProps = this._getDeclarations(pseudo);
+    // Gather all the computed properties applied by those text properties.
     let computedProps = [];
     for (const textProp of textProps) {
       computedProps = computedProps.concat(textProp.computed);
     }
 
+    // CSS Variables inherits from the normal element in case of pseudo element.
+    const variables = new Map(pseudo ? this.variablesMap.get("") : null);
+
     // Walk over the computed properties. As we see a property name
     // for the first time, mark that property's name as taken by this
     // property.
@@ -331,11 +339,13 @@ class ElementStyle {
       }
 
       let overridden;
-      if (earlier &&
-          computedProp.priority === "important" &&
-          earlier.priority !== "important" &&
-          (earlier.textProp.rule.inherited ||
-           !computedProp.textProp.rule.inherited)) {
+      if (
+        earlier &&
+        computedProp.priority === "important" &&
+        earlier.priority !== "important" &&
+        // For !important only consider rules applying to the same parent node.
+        computedProp.textProp.rule.inherited == earlier.textProp.rule.inherited
+      ) {
         // New property is higher priority. Mark the earlier property
         // overridden (which will reverse its dirty state).
         earlier._overriddenDirty = !earlier._overriddenDirty;
@@ -345,19 +355,28 @@ class ElementStyle {
         overridden = !!earlier;
       }
 
-      computedProp._overriddenDirty =
-        (!!computedProp.overridden !== overridden);
+      computedProp._overriddenDirty = !!computedProp.overridden !== overridden;
       computedProp.overridden = overridden;
 
       if (!computedProp.overridden && computedProp.textProp.enabled) {
         taken[computedProp.name] = computedProp;
 
         if (isCssVariable(computedProp.name)) {
-          this.variables.set(computedProp.name, computedProp.value);
+          variables.set(computedProp.name, computedProp.value);
         }
       }
     }
 
+    // Find the CSS variables that have been updated.
+    const previousVariablesMap = new Map(this.variablesMap.get(pseudo));
+    const changedVariableNamesSet = new Set(
+      [...variables.keys(), ...previousVariablesMap.keys()].filter(
+        k => variables.get(k) !== previousVariablesMap.get(k)
+      )
+    );
+
+    this.variablesMap.set(pseudo, variables);
+
     // For each TextProperty, mark it overridden if all of its computed
     // properties are marked overridden. Update the text property's associated
     // editor, if any. This will clear the _overriddenDirty state on all
@@ -366,7 +385,12 @@ class ElementStyle {
     for (const textProp of textProps) {
       // _updatePropertyOverridden will return true if the
       // overridden state has changed for the text property.
-      if (this._updatePropertyOverridden(textProp)) {
+      // _hasUpdatedCSSVariable will return true if the declaration contains any
+      // of the updated CSS variable names.
+      if (
+        this._updatePropertyOverridden(textProp) ||
+        this._hasUpdatedCSSVariable(textProp, changedVariableNamesSet)
+      ) {
         textProp.updateEditor();
       }
 
@@ -377,6 +401,90 @@ class ElementStyle {
     }
   }
 
+  /**
+   * Returns true if the given declaration's property value contains a CSS variable
+   * matching any of the updated CSS variable names.
+   *
+   * @param {TextProperty} declaration
+   *        A TextProperty of a rule.
+   * @param {Set<>String} variableNamesSet
+   *        A Set of CSS variable names that have been updated.
+   */
+  _hasUpdatedCSSVariable(declaration, variableNamesSet) {
+    for (const variableName of variableNamesSet) {
+      if (declaration.hasCSSVariable(variableName)) {
+        return true;
+      }
+    }
+
+    return false;
+  }
+
+  /**
+   * Helper for |this.updateDeclarations()| to mark CSS declarations as overridden.
+   *
+   * Returns an array of CSS declarations (aka TextProperty instances) from all rules
+   * applicable to the selected element ordered from more- to less-specific.
+   *
+   * If a pseudo-element type is given, restrict the result only to declarations
+   * applicable to that pseudo-element.
+   *
+   * NOTE: this method skips CSS declarations in @keyframes rules because a number of
+   * criteria such as time and animation delay need to be checked in order to determine
+   * if the property is overridden at runtime.
+   *
+   * @param  {String} pseudo
+   *         Optional pseudo-element for which to restrict marking CSS declarations as
+   *         overridden. If omitted, only declarations for regular style rules are
+   *         returned (no pseudo-element style rules).
+   *
+   * @return {Array}
+   *         Array of TextProperty instances.
+   */
+  _getDeclarations(pseudo = "") {
+    const textProps = [];
+
+    for (const rule of this.rules) {
+      // Skip @keyframes rules
+      if (rule.keyframes) {
+        continue;
+      }
+
+      // Style rules must be considered only when they have selectors that match the node.
+      // When renaming a selector, the unmatched rule lingers in the Rule view, but it no
+      // longer matches the node. This strict check avoids accidentally causing
+      // declarations to be overridden in the remaining matching rules.
+      const isStyleRule =
+        rule.pseudoElement === "" && !!rule.matchedSelectors.length;
+
+      // Style rules for pseudo-elements must always be considered, regardless if their
+      // selector matches the node. As a convenience, declarations in rules for
+      // pseudo-elements show up in a separate Pseudo-elements accordion when selecting
+      // the host node (instead of the pseudo-element node directly, which is sometimes
+      // impossible, for example with ::selection or ::first-line).
+      // Loosening the strict check on matched selectors ensures these declarations
+      // participate in the algorithm below to mark them as overridden.
+      const isPseudoElementRule =
+        rule.pseudoElement !== "" && rule.pseudoElement === pseudo;
+
+      const isElementStyle = rule.domRule.type === ELEMENT_STYLE;
+
+      const filterCondition =
+        pseudo === "" ? isStyleRule || isElementStyle : isPseudoElementRule;
+
+      // Collect all relevant CSS declarations (aka TextProperty instances).
+      if (filterCondition) {
+        for (const textProp of rule.textProps.slice(0).reverse()) {
+          if (textProp.enabled) {
+            textProps.push(textProp);
+          }
+        }
+      }
+    }
+
+    return textProps;
+  }
+
   /**
    * Adds a new declaration to the rule.
    *
@@ -391,8 +499,11 @@ class ElementStyle {
       return;
     }
 
-    const declarationsToAdd = parseNamedDeclarations(this.cssProperties.isKnown,
-      value, true);
+    const declarationsToAdd = parseNamedDeclarations(
+      this.cssProperties.isKnown,
+      value,
+      true
+    );
     if (!declarationsToAdd.length) {
       return;
     }
@@ -406,22 +517,25 @@ class ElementStyle {
    * the stylesheet.
    */
   async addNewRule() {
-    await this.pageStyle.addNewRule(this.element, this.element.pseudoClassLocks);
+    await this.pageStyle.addNewRule(
+      this.element,
+      this.element.pseudoClassLocks
+    );
   }
 
   /**
    * Given the id of the rule and the new declaration name, modifies the existing
    * declaration name to the new given value.
    *
-   * @param  {String} ruleID
+   * @param  {String} ruleId
    *         The Rule id of the given CSS declaration.
    * @param  {String} declarationId
    *         The TextProperty id for the CSS declaration.
    * @param  {String} name
    *         The new declaration name.
    */
-  async modifyDeclarationName(ruleID, declarationId, name) {
-    const rule = this.getRule(ruleID);
+  async modifyDeclarationName(ruleId, declarationId, name) {
+    const rule = this.getRule(ruleId);
     if (!rule) {
       return;
     }
@@ -460,8 +574,13 @@ class ElementStyle {
     for (const { commentOffsets, name, value, priority } of declarationsToAdd) {
       const isCommented = Boolean(commentOffsets);
       const enabled = !isCommented;
-      siblingDeclaration = rule.createProperty(name, value, priority, enabled,
-        siblingDeclaration);
+      siblingDeclaration = rule.createProperty(
+        name,
+        value,
+        priority,
+        enabled,
+        siblingDeclaration
+      );
     }
   }
 
@@ -534,12 +653,18 @@ class ElementStyle {
       return;
     }
 
-    const { declarationsToAdd, firstValue} = this._getValueAndExtraProperties(value);
-    const parsedValue = parseSingleValue(this.cssProperties.isKnown, firstValue);
+    const { declarationsToAdd, firstValue } =
+      this._getValueAndExtraProperties(value);
+    const parsedValue = parseSingleValue(
+      this.cssProperties.isKnown,
+      firstValue
+    );
 
-    if (!declarationsToAdd.length &&
-        declaration.value === parsedValue.value &&
-        declaration.priority === parsedValue.priority) {
+    if (
+      !declarationsToAdd.length &&
+      declaration.value === parsedValue.value &&
+      declaration.priority === parsedValue.priority
+    ) {
       return;
     }
 
@@ -568,7 +693,10 @@ class ElementStyle {
         return;
       }
 
-      const response = await rule.domRule.modifySelector(this.element, selector);
+      const response = await rule.domRule.modifySelector(
+        this.element,
+        selector
+      );
       const { ruleProps, isMatching } = response;
 
       if (!ruleProps) {
@@ -620,15 +748,6 @@ class ElementStyle {
     }
   }
 
-  /**
-   * Subscribes all the rules to location changes.
-   */
-  subscribeRulesToLocationChange() {
-    for (const rule of this.rules) {
-      rule.subscribeToLocationChange();
-    }
-  }
-
   /**
    * Toggles the enabled state of the given CSS declaration.
    *
@@ -674,40 +793,25 @@ class ElementStyle {
       delete computedProp._overriddenDirty;
     }
 
-    dirty = (!!prop.overridden !== overridden) || dirty;
+    dirty = !!prop.overridden !== overridden || dirty;
     prop.overridden = overridden;
     return dirty;
   }
 
- /**
-  * Returns the current value of a CSS variable; or null if the
-  * variable is not defined.
-  *
-  * @param  {String} name
-  *         The name of the variable.
-  * @return {String} the variable's value or null if the variable is
-  *         not defined.
-  */
-  getVariable(name) {
-    return this.variables.get(name);
-  }
-
   /**
-   * Handler for "stylesheet-updated", "style-changed" and "pseudoclass" events.
-   * Refreshes the list of rules on the page.
+   * Returns the current value of a CSS variable; or null if the
+   * variable is not defined.
+   *
+   * @param  {String} name
+   *         The name of the variable.
+   * @param  {String} pseudo
+   *         The pseudo-element name of the rule.
+   * @return {String} the variable's value or null if the variable is
+   *         not defined.
    */
-  async onRefresh() {
-    // Repopulate the element style once the current modifications are done.
-    const promises = [];
-    for (const rule of this.rules) {
-      if (rule._applyingModifications) {
-        promises.push(rule._applyingModifications);
-      }
-    }
-
-    await Promise.all(promises);
-    await this.populate();
-    this._changed();
+  getVariable(name, pseudo = "") {
+    const variables = this.variablesMap.get(pseudo);
+    return variables ? variables.get(name) : null;
   }
 }
 
