@@ -12,7 +12,6 @@
 #include "nscore.h"
 #include "nsIURI.h"
 #include "prprf.h"
-#include "nsErrorService.h"
 #include "netCore.h"
 #include "nsIObserverService.h"
 #include "nsXPCOM.h"
@@ -24,6 +23,7 @@
 #include "nsNetCID.h"
 #include "nsCRT.h"
 #include "nsSimpleNestedURI.h"
+#include "nsSocketTransport2.h"
 #include "nsTArray.h"
 #include "nsIConsoleService.h"
 #include "nsIUploadChannel2.h"
@@ -36,8 +36,10 @@
 #include "nsIProtocolProxyService2.h"
 #include "MainThreadUtils.h"
 #include "nsINode.h"
+#include "nsIWebTransport.h"
 #include "nsIWidget.h"
 #include "nsThreadUtils.h"
+#include "WebTransportSessionProxy.h"
 #include "mozilla/AppShutdown.h"
 #include "mozilla/LoadInfo.h"
 #include "mozilla/net/NeckoCommon.h"
@@ -63,10 +65,7 @@
 #include "mozilla/StaticPrefs_security.h"
 #include "nsNSSComponent.h"
 #include "ssl.h"
-
-#ifdef MOZ_WIDGET_GTK
-#  include "nsGIOProtocolHandler.h"
-#endif
+#include "StaticComponents.h"
 
 namespace mozilla {
 namespace net {
@@ -87,6 +86,7 @@ using mozilla::dom::ServiceWorkerDescriptor;
 #define NETWORK_CAPTIVE_PORTAL_PREF "network.captive-portal-service.enabled"
 #define WEBRTC_PREF_PREFIX "media.peerconnection."
 #define NETWORK_DNS_PREF "network.dns."
+#define FORCE_EXTERNAL_PREF_PREFIX "network.protocol-handler.external."
 
 #define MAX_RECURSION_COUNT 50
 
@@ -212,13 +212,13 @@ static const char* gCallbackPrefs[] = {
     NECKO_BUFFER_CACHE_COUNT_PREF,
     NECKO_BUFFER_CACHE_SIZE_PREF,
     NETWORK_CAPTIVE_PORTAL_PREF,
+    FORCE_EXTERNAL_PREF_PREFIX,
     nullptr,
 };
 
 static const char* gCallbackPrefsForSocketProcess[] = {
     WEBRTC_PREF_PREFIX,
     NETWORK_DNS_PREF,
-    "network.ssl_tokens_cache_enabled",
     "network.send_ODA_to_content_directly",
     "network.trr.",
     "doh-rollout.",
@@ -256,12 +256,6 @@ static const char* gCallbackSecurityPrefs[] = {
 };
 
 nsresult nsIOService::Init() {
-  // XXX hack until xpidl supports error info directly (bug 13423)
-  nsCOMPtr<nsIErrorService> errorService = nsErrorService::GetOrCreate();
-  MOZ_ALWAYS_TRUE(errorService);
-  errorService->RegisterErrorStringBundle(NS_ERROR_MODULE_NETWORK,
-                                          NECKO_MSGS_URL);
-
   SSLTokensCache::Init();
 
   InitializeCaptivePortalService();
@@ -269,9 +263,9 @@ nsresult nsIOService::Init() {
   // setup our bad port list stuff
   for (int i = 0; gBadPortList[i]; i++) {
     // We can't be accessed by another thread yet
-    PUSH_IGNORE_THREAD_SAFETY
+    MOZ_PUSH_IGNORE_THREAD_SAFETY
     mRestrictedPortList.AppendElement(gBadPortList[i]);
-    POP_THREAD_SAFETY
+    MOZ_POP_THREAD_SAFETY
   }
 
   // Further modifications to the port list come from prefs
@@ -433,6 +427,13 @@ nsresult nsIOService::InitializeCaptivePortalService() {
 nsresult nsIOService::InitializeSocketTransportService() {
   nsresult rv = NS_OK;
 
+  if (AppShutdown::IsInOrBeyond(ShutdownPhase::AppShutdownConfirmed)) {
+    LOG(
+        ("nsIOService aborting InitializeSocketTransportService because of app "
+         "shutdown"));
+    return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
+  }
+
   if (!mSocketTransportService) {
     mSocketTransportService =
         do_GetService(NS_SOCKETTRANSPORTSERVICE_CONTRACTID, &rv);
@@ -852,121 +853,65 @@ nsresult nsIOService::AsyncOnChannelRedirect(
   return NS_OK;
 }
 
-nsresult nsIOService::CacheProtocolHandler(const char* scheme,
-                                           nsIProtocolHandler* handler) {
-  MOZ_ASSERT(NS_IsMainThread());
-
-  for (unsigned int i = 0; i < NS_N(gScheme); i++) {
-    if (!nsCRT::strcasecmp(scheme, gScheme[i])) {
-      nsresult rv;
-      NS_ASSERTION(!mWeakHandler[i], "Protocol handler already cached");
-      // Make sure the handler supports weak references.
-      nsCOMPtr<nsISupportsWeakReference> factoryPtr =
-          do_QueryInterface(handler, &rv);
-      if (!factoryPtr) {
-        // Don't cache handlers that don't support weak reference as
-        // there is real danger of a circular reference.
-#ifdef DEBUG_dp
-        printf(
-            "DEBUG: %s protcol handler doesn't support weak ref. Not cached.\n",
-            scheme);
-#endif /* DEBUG_dp */
-        return NS_ERROR_FAILURE;
-      }
-      mWeakHandler[i] = do_GetWeakReference(handler);
-      return NS_OK;
-    }
-  }
-  return NS_ERROR_FAILURE;
-}
-
-nsresult nsIOService::GetCachedProtocolHandler(const char* scheme,
-                                               nsIProtocolHandler** result,
-                                               uint32_t start, uint32_t end) {
-  MOZ_ASSERT(NS_IsMainThread());
-
-  uint32_t len = end - start - 1;
-  for (unsigned int i = 0; i < NS_N(gScheme); i++) {
-    if (!mWeakHandler[i]) continue;
-
-    // handle unterminated strings
-    // start is inclusive, end is exclusive, len = end - start - 1
-    if (end ? (!nsCRT::strncasecmp(scheme + start, gScheme[i], len) &&
-               gScheme[i][len] == '\0')
-            : (!nsCRT::strcasecmp(scheme, gScheme[i]))) {
-      return CallQueryReferent(mWeakHandler[i].get(), result);
-    }
-  }
-  return NS_ERROR_FAILURE;
-}
-
-static bool UsesExternalProtocolHandler(const char* aScheme) {
-  if ("file"_ns.Equals(aScheme) || "chrome"_ns.Equals(aScheme) ||
-      "resource"_ns.Equals(aScheme)) {
+bool nsIOService::UsesExternalProtocolHandler(const nsACString& aScheme) {
+  if (aScheme == "file"_ns || aScheme == "chrome"_ns ||
+      aScheme == "resource"_ns) {
     // Don't allow file:, chrome: or resource: URIs to be handled with
     // nsExternalProtocolHandler, since internally we rely on being able to
     // use and read from these URIs.
     return false;
   }
 
-  for (const auto& forcedExternalScheme : gForcedExternalSchemes) {
-    if (!nsCRT::strcasecmp(forcedExternalScheme, aScheme)) {
+  if (aScheme == "place"_ns || aScheme == "fake-favicon-uri"_ns ||
+      aScheme == "favicon"_ns || aScheme == "moz-nullprincipal"_ns) {
+    // Force place: fake-favicon-uri: favicon: and moz-nullprincipal: URIs to be
+    // handled with nsExternalProtocolHandler, and not with a dynamically
+    // registered handler.
+    return true;
+  }
+
+  // If prefs configure the URI to be handled externally, do so.
+  for (const auto& scheme : mForceExternalSchemes) {
+    if (aScheme == scheme) {
       return true;
     }
   }
+  return false;
+}
 
-  nsAutoCString pref("network.protocol-handler.external.");
-  pref += aScheme;
+ProtocolHandlerInfo nsIOService::LookupProtocolHandler(
+    const nsACString& aScheme) {
+  // Look-ups are ASCII-case-insensitive, so lower-case the string before
+  // continuing.
+  nsAutoCString scheme(aScheme);
+  ToLowerCase(scheme);
 
-  return Preferences::GetBool(pref.get(), false);
+  // NOTE: If we could get rid of mForceExternalSchemes (or prevent them from
+  // disabling static protocols), we could avoid locking mLock until we need to
+  // check `mRuntimeProtocolHandlers.
+  AutoReadLock lock(mLock);
+  if (!UsesExternalProtocolHandler(scheme)) {
+    // Try the static protocol handler first - they cannot be overridden by
+    // dynamic protocols.
+    if (const xpcom::StaticProtocolHandler* handler =
+            xpcom::StaticProtocolHandler::Lookup(scheme)) {
+      return ProtocolHandlerInfo(*handler);
+    }
+    if (auto handler = mRuntimeProtocolHandlers.Lookup(scheme)) {
+      return ProtocolHandlerInfo(handler.Data());
+    }
+  }
+  return ProtocolHandlerInfo(xpcom::StaticProtocolHandler::Default());
 }
 
 NS_IMETHODIMP
 nsIOService::GetProtocolHandler(const char* scheme,
                                 nsIProtocolHandler** result) {
-  nsresult rv;
-
+  AssertIsOnMainThread();
   NS_ENSURE_ARG_POINTER(scheme);
-  // XXX we may want to speed this up by introducing our own protocol
-  // scheme -> protocol handler mapping, avoiding the string manipulation
-  // and service manager stuff
 
-  rv = GetCachedProtocolHandler(scheme, result);
-  if (NS_SUCCEEDED(rv)) return rv;
-
-  if (scheme[0] != '\0' && !UsesExternalProtocolHandler(scheme)) {
-    nsAutoCString contractID(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX);
-    contractID += scheme;
-    ToLowerCase(contractID);
-
-    rv = CallGetService(contractID.get(), result);
-    if (NS_SUCCEEDED(rv)) {
-      CacheProtocolHandler(scheme, *result);
-      return rv;
-    }
-
-#ifdef MOZ_WIDGET_GTK
-    // check to see whether GVFS can handle this URI scheme. otherwise, we
-    // failover to using the default protocol handler.
-
-    RefPtr<nsGIOProtocolHandler> gioHandler =
-        nsGIOProtocolHandler::GetSingleton();
-    if (gioHandler->IsSupportedProtocol(nsCString(scheme))) {
-      gioHandler.forget(result);
-      return NS_OK;
-    }
-#endif
-  }
-
-  // Okay we don't have a protocol handler to handle this url type, so use
-  // the default protocol handler.  This will cause urls to get dispatched
-  // out to the OS ('cause we can't do anything with them) when we try to
-  // read from a channel created by the default protocol handler.
-
-  rv = CallGetService(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX "default", result);
-  if (NS_FAILED(rv)) return NS_ERROR_UNKNOWN_PROTOCOL;
-
-  return rv;
+  *result = LookupProtocolHandler(nsDependentCString(scheme)).Handler().take();
+  return *result ? NS_OK : NS_ERROR_UNKNOWN_PROTOCOL;
 }
 
 NS_IMETHODIMP
@@ -1022,19 +967,32 @@ nsIOService::HostnameIsSharedIPAddress(nsIURI* aURI, bool* aResult) {
 
 NS_IMETHODIMP
 nsIOService::GetProtocolFlags(const char* scheme, uint32_t* flags) {
-  nsCOMPtr<nsIProtocolHandler> handler;
-  nsresult rv = GetProtocolHandler(scheme, getter_AddRefs(handler));
-  if (NS_FAILED(rv)) return rv;
+  NS_ENSURE_ARG_POINTER(scheme);
 
-  // We can't call DoGetProtocolFlags here because we don't have a URI. This
-  // API is used by (and only used by) extensions, which is why it's still
-  // around. Calling this on a scheme with dynamic flags will throw.
-  rv = handler->GetProtocolFlags(flags);
-#if !IS_ORIGIN_IS_FULL_SPEC_DEFINED
-  MOZ_RELEASE_ASSERT(!(*flags & nsIProtocolHandler::ORIGIN_IS_FULL_SPEC),
-                     "ORIGIN_IS_FULL_SPEC is unsupported but used");
-#endif
-  return rv;
+  *flags =
+      LookupProtocolHandler(nsDependentCString(scheme)).StaticProtocolFlags();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsIOService::GetDynamicProtocolFlags(nsIURI* uri, uint32_t* flags) {
+  AssertIsOnMainThread();
+  NS_ENSURE_ARG(uri);
+
+  nsAutoCString scheme;
+  nsresult rv = uri->GetScheme(scheme);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return LookupProtocolHandler(scheme).DynamicProtocolFlags(uri, flags);
+}
+
+NS_IMETHODIMP
+nsIOService::GetDefaultPort(const char* scheme, int32_t* defaultPort) {
+  NS_ENSURE_ARG_POINTER(scheme);
+
+  *defaultPort =
+      LookupProtocolHandler(nsDependentCString(scheme)).DefaultPort();
+  return NS_OK;
 }
 
 class AutoIncrement {
@@ -1110,13 +1068,14 @@ nsresult nsIOService::NewChannelFromURIWithClientAndController(
     const Maybe<ClientInfo>& aLoadingClientInfo,
     const Maybe<ServiceWorkerDescriptor>& aController, uint32_t aSecurityFlags,
     nsContentPolicyType aContentPolicyType, uint32_t aSandboxFlags,
-    nsIChannel** aResult) {
+    bool aSkipCheckForBrokenURLOrZeroSized, nsIChannel** aResult) {
   return NewChannelFromURIWithProxyFlagsInternal(
       aURI,
       nullptr,  // aProxyURI
       0,        // aProxyFlags
       aLoadingNode, aLoadingPrincipal, aTriggeringPrincipal, aLoadingClientInfo,
-      aController, aSecurityFlags, aContentPolicyType, aSandboxFlags, aResult);
+      aController, aSecurityFlags, aContentPolicyType, aSandboxFlags,
+      aSkipCheckForBrokenURLOrZeroSized, aResult);
 }
 
 NS_IMETHODIMP
@@ -1135,10 +1094,11 @@ nsresult nsIOService::NewChannelFromURIWithProxyFlagsInternal(
     const Maybe<ClientInfo>& aLoadingClientInfo,
     const Maybe<ServiceWorkerDescriptor>& aController, uint32_t aSecurityFlags,
     nsContentPolicyType aContentPolicyType, uint32_t aSandboxFlags,
-    nsIChannel** result) {
+    bool aSkipCheckForBrokenURLOrZeroSized, nsIChannel** result) {
   nsCOMPtr<nsILoadInfo> loadInfo = new LoadInfo(
       aLoadingPrincipal, aTriggeringPrincipal, aLoadingNode, aSecurityFlags,
-      aContentPolicyType, aLoadingClientInfo, aController, aSandboxFlags);
+      aContentPolicyType, aLoadingClientInfo, aController, aSandboxFlags,
+      aSkipCheckForBrokenURLOrZeroSized);
   return NewChannelFromURIWithProxyFlagsInternal(aURI, aProxyURI, aProxyFlags,
                                                  loadInfo, result);
 }
@@ -1160,10 +1120,6 @@ nsresult nsIOService::NewChannelFromURIWithProxyFlagsInternal(
   rv = GetProtocolHandler(scheme.get(), getter_AddRefs(handler));
   if (NS_FAILED(rv)) return rv;
 
-  uint32_t protoFlags;
-  rv = handler->DoGetProtocolFlags(aURI, &protoFlags);
-  if (NS_FAILED(rv)) return rv;
-
   nsCOMPtr<nsIChannel> channel;
   nsCOMPtr<nsIProxiedProtocolHandler> pph = do_QueryInterface(handler);
   if (pph) {
@@ -1224,7 +1180,7 @@ nsIOService::NewChannelFromURIWithProxyFlags(
       aURI, aProxyURI, aProxyFlags, aLoadingNode, aLoadingPrincipal,
       aTriggeringPrincipal, Maybe<ClientInfo>(),
       Maybe<ServiceWorkerDescriptor>(), aSecurityFlags, aContentPolicyType, 0,
-      result);
+      /* aSkipCheckForBrokenURLOrZeroSized = */ false, result);
 }
 
 NS_IMETHODIMP
@@ -1245,6 +1201,18 @@ nsIOService::NewChannel(const nsACString& aSpec, const char* aCharset,
                            aContentPolicyType, result);
 }
 
+NS_IMETHODIMP
+nsIOService::NewWebTransport(nsIWebTransport** result) {
+  if (!XRE_IsParentProcess()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  nsCOMPtr<nsIWebTransport> webTransport = new WebTransportSessionProxy();
+
+  webTransport.forget(result);
+  return NS_OK;
+}
+
 bool nsIOService::IsLinkUp() {
   InitializeNetworkLinkService();
 
@@ -1456,7 +1424,7 @@ nsIOService::AllowPort(int32_t inPort, const char* scheme, bool* _retval) {
 
   nsTArray<int32_t> restrictedPortList;
   {
-    MutexAutoLock lock(mMutex);
+    AutoReadLock lock(mLock);
     restrictedPortList.Assign(mRestrictedPortList);
   }
   // first check to see if the port is in our blacklist:
@@ -1550,13 +1518,30 @@ void nsIOService::PrefsChanged(const char* pref) {
       }
     }
   }
+
+  if (!pref || strncmp(pref, FORCE_EXTERNAL_PREF_PREFIX,
+                       strlen(FORCE_EXTERNAL_PREF_PREFIX)) == 0) {
+    nsTArray<nsCString> prefs;
+    if (nsIPrefBranch* prefRootBranch = Preferences::GetRootBranch()) {
+      prefRootBranch->GetChildList(FORCE_EXTERNAL_PREF_PREFIX, prefs);
+    }
+    nsTArray<nsCString> forceExternalSchemes;
+    for (const auto& pref : prefs) {
+      if (Preferences::GetBool(pref.get(), false)) {
+        forceExternalSchemes.AppendElement(
+            Substring(pref, strlen(FORCE_EXTERNAL_PREF_PREFIX)));
+      }
+    }
+    AutoWriteLock lock(mLock);
+    mForceExternalSchemes = std::move(forceExternalSchemes);
+  }
 }
 
 void nsIOService::ParsePortList(const char* pref, bool remove) {
   nsAutoCString portList;
   nsTArray<int32_t> restrictedPortList;
   {
-    MutexAutoLock lock(mMutex);
+    AutoWriteLock lock(mLock);
     restrictedPortList.Assign(std::move(mRestrictedPortList));
   }
   // Get a pref string and chop it up into a list of ports.
@@ -1597,7 +1582,7 @@ void nsIOService::ParsePortList(const char* pref, bool remove) {
     }
   }
 
-  MutexAutoLock lock(mMutex);
+  AutoWriteLock lock(mLock);
   mRestrictedPortList.Assign(std::move(restrictedPortList));
 }
 
@@ -1703,7 +1688,14 @@ nsIOService::Observe(nsISupports* subject, const char* topic,
     if (IsSocketProcessChild()) {
       Preferences::UnregisterCallbacks(nsIOService::OnTLSPrefChange,
                                        gCallbackSecurityPrefs, this);
-      NSSShutdownForSocketProcess();
+      PrepareForShutdownInSocketProcess();
+    }
+
+    // We're in XPCOM shutdown now. Unregister any dynamic protocol handlers
+    // after this point to avoid leaks.
+    {
+      AutoWriteLock lock(mLock);
+      mRuntimeProtocolHandlers.Clear();
     }
   } else if (!strcmp(topic, NS_NETWORK_LINK_TOPIC)) {
     OnNetworkLinkEvent(NS_ConvertUTF16toUTF8(data).get());
@@ -1747,13 +1739,16 @@ nsIOService::ProtocolHasFlags(nsIURI* uri, uint32_t flags, bool* result) {
   nsresult rv = uri->GetScheme(scheme);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // Grab the protocol flags from the URI.
+  auto handler = LookupProtocolHandler(scheme);
+
   uint32_t protocolFlags;
-  nsCOMPtr<nsIProtocolHandler> handler;
-  rv = GetProtocolHandler(scheme.get(), getter_AddRefs(handler));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = handler->DoGetProtocolFlags(uri, &protocolFlags);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (flags & nsIProtocolHandler::DYNAMIC_URI_FLAGS) {
+    AssertIsOnMainThread();
+    rv = handler.DynamicProtocolFlags(uri, &protocolFlags);
+    NS_ENSURE_SUCCESS(rv, rv);
+  } else {
+    protocolFlags = handler.StaticProtocolFlags();
+  }
 
   *result = (protocolFlags & flags) == flags;
   return NS_OK;
@@ -1928,12 +1923,16 @@ class IOServiceProxyCallback final : public nsIProtocolProxyCallback {
   NS_DECL_NSIPROTOCOLPROXYCALLBACK
 
   IOServiceProxyCallback(nsIInterfaceRequestor* aCallbacks,
-                         nsIOService* aIOService)
-      : mCallbacks(aCallbacks), mIOService(aIOService) {}
+                         nsIOService* aIOService,
+                         Maybe<OriginAttributes>&& aOriginAttributes)
+      : mCallbacks(aCallbacks),
+        mIOService(aIOService),
+        mOriginAttributes(std::move(aOriginAttributes)) {}
 
  private:
   RefPtr<nsIInterfaceRequestor> mCallbacks;
   RefPtr<nsIOService> mIOService;
+  Maybe<OriginAttributes> mOriginAttributes;
 };
 
 NS_IMPL_ISUPPORTS(IOServiceProxyCallback, nsIProtocolProxyCallback)
@@ -1973,18 +1972,22 @@ IOServiceProxyCallback::OnProxyAvailable(nsICancelable* request,
 
   nsLoadFlags loadFlags = 0;
   channel->GetLoadFlags(&loadFlags);
-  if (loadFlags & nsIRequest::LOAD_ANONYMOUS) {
-    speculativeHandler->SpeculativeAnonymousConnect(uri, principal, mCallbacks);
+  bool anonymous = !!(loadFlags & nsIRequest::LOAD_ANONYMOUS);
+  if (mOriginAttributes) {
+    speculativeHandler->SpeculativeConnectWithOriginAttributesNative(
+        uri, std::move(mOriginAttributes.ref()), mCallbacks, anonymous);
   } else {
-    speculativeHandler->SpeculativeConnect(uri, principal, mCallbacks);
+    speculativeHandler->SpeculativeConnect(uri, principal, mCallbacks,
+                                           anonymous);
   }
 
   return NS_OK;
 }
 
 nsresult nsIOService::SpeculativeConnectInternal(
-    nsIURI* aURI, nsIPrincipal* aPrincipal, nsIInterfaceRequestor* aCallbacks,
-    bool aAnonymous) {
+    nsIURI* aURI, nsIPrincipal* aPrincipal,
+    Maybe<OriginAttributes>&& aOriginAttributes,
+    nsIInterfaceRequestor* aCallbacks, bool aAnonymous) {
   NS_ENSURE_ARG(aURI);
 
   if (!aURI->SchemeIs("http") && !aURI->SchemeIs("https")) {
@@ -1993,7 +1996,8 @@ nsresult nsIOService::SpeculativeConnectInternal(
   }
 
   if (IsNeckoChild()) {
-    gNeckoChild->SendSpeculativeConnect(aURI, aPrincipal, aAnonymous);
+    gNeckoChild->SendSpeculativeConnect(
+        aURI, aPrincipal, std::move(aOriginAttributes), aAnonymous);
     return NS_OK;
   }
 
@@ -2007,12 +2011,18 @@ nsresult nsIOService::SpeculativeConnectInternal(
 
   nsCOMPtr<nsIPrincipal> loadingPrincipal = aPrincipal;
 
-  MOZ_ASSERT(aPrincipal, "We expect passing a principal here.");
+  MOZ_ASSERT(aPrincipal || aOriginAttributes,
+             "We expect passing a principal or OriginAttributes here.");
 
-  if (!aPrincipal) {
+  if (!aPrincipal && !aOriginAttributes) {
     return NS_ERROR_INVALID_ARG;
   }
 
+  if (aOriginAttributes) {
+    loadingPrincipal =
+        BasePrincipal::CreateContentPrincipal(aURI, aOriginAttributes.ref());
+  }
+
   // XXX Bug 1724080: Avoid TCP connections on port 80 when https-only
   // or https-first is enabled. Let's create a dummy loadinfo which we
   // only use to determine whether we need ot upgrade the speculative
@@ -2058,8 +2068,8 @@ nsresult nsIOService::SpeculativeConnectInternal(
   }
 
   nsCOMPtr<nsICancelable> cancelable;
-  RefPtr<IOServiceProxyCallback> callback =
-      new IOServiceProxyCallback(aCallbacks, this);
+  RefPtr<IOServiceProxyCallback> callback = new IOServiceProxyCallback(
+      aCallbacks, this, std::move(aOriginAttributes));
   nsCOMPtr<nsIProtocolProxyService2> pps2 = do_QueryInterface(pps);
   if (pps2) {
     return pps2->AsyncResolve2(channel, 0, callback, nullptr,
@@ -2071,14 +2081,33 @@ nsresult nsIOService::SpeculativeConnectInternal(
 
 NS_IMETHODIMP
 nsIOService::SpeculativeConnect(nsIURI* aURI, nsIPrincipal* aPrincipal,
-                                nsIInterfaceRequestor* aCallbacks) {
-  return SpeculativeConnectInternal(aURI, aPrincipal, aCallbacks, false);
+                                nsIInterfaceRequestor* aCallbacks,
+                                bool aAnonymous) {
+  return SpeculativeConnectInternal(aURI, aPrincipal, Nothing(), aCallbacks,
+                                    aAnonymous);
 }
 
-NS_IMETHODIMP
-nsIOService::SpeculativeAnonymousConnect(nsIURI* aURI, nsIPrincipal* aPrincipal,
-                                         nsIInterfaceRequestor* aCallbacks) {
-  return SpeculativeConnectInternal(aURI, aPrincipal, aCallbacks, true);
+NS_IMETHODIMP nsIOService::SpeculativeConnectWithOriginAttributes(
+    nsIURI* aURI, JS::Handle<JS::Value> aOriginAttributes,
+    nsIInterfaceRequestor* aCallbacks, bool aAnonymous, JSContext* aCx) {
+  OriginAttributes attrs;
+  if (!aOriginAttributes.isObject() || !attrs.Init(aCx, aOriginAttributes)) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  SpeculativeConnectWithOriginAttributesNative(aURI, std::move(attrs),
+                                               aCallbacks, aAnonymous);
+  return NS_OK;
+}
+
+NS_IMETHODIMP_(void)
+nsIOService::SpeculativeConnectWithOriginAttributesNative(
+    nsIURI* aURI, OriginAttributes&& aOriginAttributes,
+    nsIInterfaceRequestor* aCallbacks, bool aAnonymous) {
+  Maybe<OriginAttributes> originAttributes;
+  originAttributes.emplace(aOriginAttributes);
+  Unused << SpeculativeConnectInternal(
+      aURI, nullptr, std::move(originAttributes), aCallbacks, aAnonymous);
 }
 
 NS_IMETHODIMP
@@ -2113,5 +2142,60 @@ nsIOService::GetSocketProcessId(uint64_t* aPid) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsIOService::RegisterProtocolHandler(const nsACString& aScheme,
+                                     nsIProtocolHandler* aHandler,
+                                     uint32_t aProtocolFlags,
+                                     int32_t aDefaultPort) {
+  if (mShutdown) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  if (aScheme.IsEmpty()) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  nsAutoCString scheme(aScheme);
+  ToLowerCase(scheme);
+
+  AutoWriteLock lock(mLock);
+  return mRuntimeProtocolHandlers.WithEntryHandle(scheme, [&](auto&& entry) {
+    if (entry) {
+      NS_WARNING("Cannot override an existing dynamic protocol handler");
+      return NS_ERROR_FACTORY_EXISTS;
+    }
+    if (xpcom::StaticProtocolHandler::Lookup(scheme)) {
+      NS_WARNING("Cannot override an existing static protocol handler");
+      return NS_ERROR_FACTORY_EXISTS;
+    }
+    nsMainThreadPtrHandle<nsIProtocolHandler> handler(
+        new nsMainThreadPtrHolder<nsIProtocolHandler>("RuntimeProtocolHandler",
+                                                      aHandler));
+    entry.Insert(RuntimeProtocolHandler{
+        .mHandler = std::move(handler),
+        .mProtocolFlags = aProtocolFlags,
+        .mDefaultPort = aDefaultPort,
+    });
+    return NS_OK;
+  });
+}
+
+NS_IMETHODIMP
+nsIOService::UnregisterProtocolHandler(const nsACString& aScheme) {
+  if (mShutdown) {
+    return NS_OK;
+  }
+  if (aScheme.IsEmpty()) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  nsAutoCString scheme(aScheme);
+  ToLowerCase(scheme);
+
+  AutoWriteLock lock(mLock);
+  return mRuntimeProtocolHandlers.Remove(scheme)
+             ? NS_OK
+             : NS_ERROR_FACTORY_NOT_REGISTERED;
+}
+
 }  // namespace net
 }  // namespace mozilla
