@@ -39,12 +39,7 @@ using namespace mozilla::ipc;
 namespace mozilla {
 namespace net {
 
-// Pref string constants
-static const char kCookieMoveIntervalSecs[] =
-    "network.cookie.move.interval_sec";
-
 static StaticRefPtr<CookieServiceChild> gCookieChildService;
-static uint32_t gMoveCookiesIntervalSeconds = 10;
 
 already_AddRefed<CookieServiceChild> CookieServiceChild::GetSingleton() {
   if (!gCookieChildService) {
@@ -55,8 +50,8 @@ already_AddRefed<CookieServiceChild> CookieServiceChild::GetSingleton() {
   return do_AddRef(gCookieChildService);
 }
 
-NS_IMPL_ISUPPORTS(CookieServiceChild, nsICookieService, nsIObserver,
-                  nsITimerCallback, nsINamed, nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(CookieServiceChild, nsICookieService,
+                  nsISupportsWeakReference)
 
 CookieServiceChild::CookieServiceChild() {
   NS_ASSERTION(IsNeckoChild(), "not a child process");
@@ -79,50 +74,6 @@ CookieServiceChild::CookieServiceChild() {
 
   mTLDService = do_GetService(NS_EFFECTIVETLDSERVICE_CONTRACTID);
   NS_ASSERTION(mTLDService, "couldn't get TLDService");
-
-  // Init our prefs and observer.
-  nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  NS_WARNING_ASSERTION(prefBranch, "no prefservice");
-  if (prefBranch) {
-    prefBranch->AddObserver(kCookieMoveIntervalSecs, this, true);
-    PrefChanged(prefBranch);
-  }
-
-  nsCOMPtr<nsIObserverService> observerService = services::GetObserverService();
-  if (observerService) {
-    observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, false);
-  }
-}
-
-void CookieServiceChild::MoveCookies() {
-  TimeStamp start = TimeStamp::Now();
-  for (const auto& cookiesList : mCookiesMap.Values()) {
-    CookiesList newCookiesList;
-    for (uint32_t i = 0; i < cookiesList->Length(); ++i) {
-      Cookie* cookie = cookiesList->ElementAt(i);
-      RefPtr<Cookie> newCookie = cookie->Clone();
-      newCookiesList.AppendElement(newCookie);
-    }
-    *cookiesList = std::move(newCookiesList);
-  }
-
-  Telemetry::AccumulateTimeDelta(Telemetry::COOKIE_TIME_MOVING_MS, start);
-}
-
-NS_IMETHODIMP
-CookieServiceChild::Notify(nsITimer* aTimer) {
-  if (aTimer == mCookieTimer) {
-    MoveCookies();
-  } else {
-    MOZ_CRASH("Unknown timer");
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-CookieServiceChild::GetName(nsACString& aName) {
-  aName.AssignLiteral("CookieServiceChild");
-  return NS_OK;
 }
 
 CookieServiceChild::~CookieServiceChild() { gCookieChildService = nullptr; }
@@ -191,6 +142,13 @@ IPCResult CookieServiceChild::RecvAddCookie(const CookieStruct& aCookie,
                                             const OriginAttributes& aAttrs) {
   RefPtr<Cookie> cookie = Cookie::Create(aCookie, aAttrs);
   RecordDocumentCookie(cookie, aAttrs);
+
+  // signal test code to check their cookie list
+  nsCOMPtr<nsIObserverService> obsService = services::GetObserverService();
+  if (obsService) {
+    obsService->NotifyObservers(nullptr, "cookie-content-filter-test", nullptr);
+  }
+
   return IPC_OK();
 }
 
@@ -216,25 +174,6 @@ IPCResult CookieServiceChild::RecvTrackCookiesLoad(
   return IPC_OK();
 }
 
-void CookieServiceChild::PrefChanged(nsIPrefBranch* aPrefBranch) {
-  int32_t val;
-  if (NS_SUCCEEDED(aPrefBranch->GetIntPref(kCookieMoveIntervalSecs, &val))) {
-    gMoveCookiesIntervalSeconds = clamped<uint32_t>(val, 0, 3600);
-    if (gMoveCookiesIntervalSeconds && !mCookieTimer) {
-      NS_NewTimerWithCallback(getter_AddRefs(mCookieTimer), this,
-                              gMoveCookiesIntervalSeconds * 1000,
-                              nsITimer::TYPE_REPEATING_SLACK_LOW_PRIORITY);
-    }
-    if (!gMoveCookiesIntervalSeconds && mCookieTimer) {
-      mCookieTimer->Cancel();
-      mCookieTimer = nullptr;
-    }
-    if (mCookieTimer) {
-      mCookieTimer->SetDelay(gMoveCookiesIntervalSeconds * 1000);
-    }
-  }
-}
-
 uint32_t CookieServiceChild::CountCookiesFromHashTable(
     const nsACString& aBaseDomain, const OriginAttributes& aOriginAttrs) {
   CookiesList* cookiesList = nullptr;
@@ -310,31 +249,6 @@ void CookieServiceChild::RecordDocumentCookie(Cookie* aCookie,
   cookiesList->AppendElement(aCookie);
 }
 
-NS_IMETHODIMP
-CookieServiceChild::Observe(nsISupports* aSubject, const char* aTopic,
-                            const char16_t* /*aData*/) {
-  if (!strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)) {
-    if (mCookieTimer) {
-      mCookieTimer->Cancel();
-      mCookieTimer = nullptr;
-    }
-    nsCOMPtr<nsIObserverService> observerService =
-        services::GetObserverService();
-    if (observerService) {
-      observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
-    }
-  } else if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
-    nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(aSubject);
-    if (prefBranch) {
-      PrefChanged(prefBranch);
-    }
-  } else {
-    MOZ_ASSERT(false, "unexpected topic!");
-  }
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 CookieServiceChild::GetCookieStringFromDocument(dom::Document* aDocument,
                                                 nsACString& aCookieString) {
@@ -342,7 +256,7 @@ CookieServiceChild::GetCookieStringFromDocument(dom::Document* aDocument,
 
   aCookieString.Truncate();
 
-  nsCOMPtr<nsIPrincipal> principal = aDocument->EffectiveStoragePrincipal();
+  nsCOMPtr<nsIPrincipal> principal = aDocument->EffectiveCookiePrincipal();
 
   if (!CookieCommons::IsSchemeSupported(principal)) {
     return NS_OK;
@@ -407,7 +321,7 @@ CookieServiceChild::GetCookieStringFromDocument(dom::Document* aDocument,
       continue;
     }
 
-    // if the cookie is secure and the host scheme isn't, we can't send it
+    // do not display the cookie if it is secure and the host scheme isn't
     if (cookie->IsSecure() && !isPotentiallyTrustworthy) {
       continue;
     }
@@ -492,17 +406,29 @@ CookieServiceChild::SetCookieStringFromDocument(
 
   if (cookies) {
     // We need to see if the cookie we're setting would overwrite an httponly
-    // one. This would not affect anything we send over the net (those come
-    // from the parent, which already checks this), but script could see an
-    // inconsistent view of things.
+    // or a secure one. This would not affect anything we send over the net
+    // (those come from the parent, which already checks this),
+    // but script could see an inconsistent view of things.
+
+    nsCOMPtr<nsIPrincipal> principal = aDocument->EffectiveCookiePrincipal();
+    bool isPotentiallyTrustworthy =
+        principal->GetIsOriginPotentiallyTrustworthy();
+
     for (uint32_t i = 0; i < cookies->Length(); ++i) {
       RefPtr<Cookie> existingCookie = cookies->ElementAt(i);
       if (existingCookie->Name().Equals(cookie->Name()) &&
           existingCookie->Host().Equals(cookie->Host()) &&
-          existingCookie->Path().Equals(cookie->Path()) &&
-          existingCookie->IsHttpOnly()) {
+          existingCookie->Path().Equals(cookie->Path())) {
         // Can't overwrite an httponly cookie from a script context.
-        return NS_OK;
+        if (existingCookie->IsHttpOnly()) {
+          return NS_OK;
+        }
+
+        // prevent insecure cookie from overwriting a secure one in insecure
+        // context.
+        if (existingCookie->IsSecure() && !isPotentiallyTrustworthy) {
+          return NS_OK;
+        }
       }
     }
   }
