@@ -49,12 +49,11 @@ nsresult nsAboutCache::Channel::Init(nsIURI* aURI, nsILoadInfo* aLoadInfo) {
   mCancel = false;
 
   nsCOMPtr<nsIInputStream> inputStream;
-  rv = NS_NewPipe(getter_AddRefs(inputStream), getter_AddRefs(mStream), 16384,
-                  (uint32_t)-1,
-                  true,  // non-blocking input
-                  false  // blocking output
+  NS_NewPipe(getter_AddRefs(inputStream), getter_AddRefs(mStream), 16384,
+             (uint32_t)-1,
+             true,  // non-blocking input
+             false  // blocking output
   );
-  if (NS_FAILED(rv)) return rv;
 
   nsAutoCString storageName;
   rv = ParseURI(aURI, storageName);
@@ -119,7 +118,7 @@ NS_IMETHODIMP nsAboutCache::Channel::AsyncOpen(nsIStreamListener* aListener) {
   rv = VisitNextStorage();
   if (NS_FAILED(rv)) return rv;
 
-  rv = NS_MaybeOpenChannelUsingAsyncOpen(mChannel, aListener);
+  rv = mChannel->AsyncOpen(aListener);
   if (NS_FAILED(rv)) return rv;
 
   return NS_OK;
@@ -326,12 +325,10 @@ nsAboutCache::Channel::OnCacheStorageInfo(uint32_t aEntryCount,
 }
 
 NS_IMETHODIMP
-nsAboutCache::Channel::OnCacheEntryInfo(nsIURI* aURI,
-                                        const nsACString& aIdEnhance,
-                                        int64_t aDataSize, int32_t aFetchCount,
-                                        uint32_t aLastModified,
-                                        uint32_t aExpirationTime, bool aPinned,
-                                        nsILoadContextInfo* aInfo) {
+nsAboutCache::Channel::OnCacheEntryInfo(
+    nsIURI* aURI, const nsACString& aIdEnhance, int64_t aDataSize,
+    int64_t aAltDataSize, uint32_t aFetchCount, uint32_t aLastModified,
+    uint32_t aExpirationTime, bool aPinned, nsILoadContextInfo* aInfo) {
   // We need mStream for this
   if (!mStream || mCancel) {
     // Returning a failure from this callback stops the iteration
@@ -345,6 +342,7 @@ nsAboutCache::Channel::OnCacheEntryInfo(nsIURI* aURI,
         "  <colgroup>\n"
         "   <col id=\"col-key\">\n"
         "   <col id=\"col-dataSize\">\n"
+        "   <col id=\"col-altDataSize\">\n"
         "   <col id=\"col-fetchCount\">\n"
         "   <col id=\"col-lastModified\">\n"
         "   <col id=\"col-expires\">\n"
@@ -354,6 +352,7 @@ nsAboutCache::Channel::OnCacheEntryInfo(nsIURI* aURI,
         "    <tr>\n"
         "      <th>Key</th>\n"
         "      <th>Data size</th>\n"
+        "      <th>Alternative Data size</th>\n"
         "      <th>Fetch count</th>\n"
         "      <th>Last Modifed</th>\n"
         "      <th>Expires</th>\n"
@@ -413,6 +412,11 @@ nsAboutCache::Channel::OnCacheEntryInfo(nsIURI* aURI,
   mBuffer.AppendInt(aDataSize);
   mBuffer.AppendLiteral(" bytes</td>\n");
 
+  // Length of alternative content
+  mBuffer.AppendLiteral("    <td>");
+  mBuffer.AppendInt(aAltDataSize);
+  mBuffer.AppendLiteral(" bytes</td>\n");
+
   // Number of accesses
   mBuffer.AppendLiteral("    <td>");
   mBuffer.AppendInt(aFetchCount);
