@@ -73,7 +73,8 @@ class nsCORSListenerProxy final : public nsIStreamListener,
                                     bool aPrivateBrowsing,
                                     bool aFromChromeContext,
                                     const nsAString& aMessage,
-                                    const nsACString& aCategory);
+                                    const nsACString& aCategory,
+                                    bool aIsWarning = false);
 
  private:
   // Only HttpChannelParent can call RemoveFromCorsPreflightCache
@@ -112,6 +113,7 @@ class nsCORSListenerProxy final : public nsIStreamListener,
   // an http: request to https: in nsHttpChannel::Connect() and hence
   // a request might not be marked as cross site request based on that promise.
   bool mHasBeenCrossSite;
+  bool mIsRedirect = false;
   // Under e10s, logging happens in the child process. Keep a reference to the
   // creator nsIHttpChannel in order to find the way back to the child. Released
   // in OnStopRequest().
