@@ -10,6 +10,7 @@
 #include "nsContentSecurityManager.h"
 #include "nsContentUtils.h"
 #include "nsHttpChannel.h"
+#include "nsIExternalProtocolHandler.h"
 #include "nsIHttpHeaderVisitor.h"
 #include "nsIIOService.h"
 #include "nsIInputStreamChannel.h"
@@ -44,6 +45,26 @@ NS_INTERFACE_MAP_BEGIN(nsViewSourceChannel)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIViewSourceChannel)
 NS_INTERFACE_MAP_END
 
+static nsresult WillUseExternalProtocolHandler(nsIIOService* aIOService,
+                                               const char* aScheme) {
+  nsCOMPtr<nsIProtocolHandler> handler;
+  nsresult rv =
+      aIOService->GetProtocolHandler(aScheme, getter_AddRefs(handler));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCOMPtr<nsIExternalProtocolHandler> externalHandler =
+      do_QueryInterface(handler);
+  // We should not allow view-source to open any external app.
+  if (externalHandler) {
+    NS_WARNING(nsPrintfCString("blocking view-source:%s:", aScheme).get());
+    return NS_ERROR_MALFORMED_URI;
+  }
+
+  return NS_OK;
+}
+
 nsresult nsViewSourceChannel::Init(nsIURI* uri, nsILoadInfo* aLoadInfo) {
   mOriginalURI = uri;
 
@@ -64,6 +85,11 @@ nsresult nsViewSourceChannel::Init(nsIURI* uri, nsILoadInfo* aLoadInfo) {
     return NS_ERROR_INVALID_ARG;
   }
 
+  rv = WillUseExternalProtocolHandler(pService, scheme.get());
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
   nsCOMPtr<nsIURI> newChannelURI;
   rv = pService->NewURI(path, nullptr, nullptr, getter_AddRefs(newChannelURI));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -237,6 +263,20 @@ nsViewSourceChannel::GetStatus(nsresult* status) {
   return mChannel->GetStatus(status);
 }
 
+NS_IMETHODIMP nsViewSourceChannel::SetCanceledReason(
+    const nsACString& aReason) {
+  return nsIViewSourceChannel::SetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP nsViewSourceChannel::GetCanceledReason(nsACString& aReason) {
+  return nsIViewSourceChannel::GetCanceledReasonImpl(aReason);
+}
+
+NS_IMETHODIMP nsViewSourceChannel::CancelWithReason(nsresult aStatus,
+                                                    const nsACString& aReason) {
+  return nsIViewSourceChannel::CancelWithReasonImpl(aStatus, aReason);
+}
+
 NS_IMETHODIMP
 nsViewSourceChannel::Cancel(nsresult status) {
   NS_ENSURE_TRUE(mChannel, NS_ERROR_FAILURE);
@@ -431,7 +471,8 @@ nsViewSourceChannel::GetContentType(nsACString& aContentType) {
     // content decoder will then kick in automatically, and it
     // will call our SetOriginalContentType method instead of our
     // SetContentType method to set the type it determines.
-    if (!contentType.EqualsLiteral(UNKNOWN_CONTENT_TYPE)) {
+    if (!contentType.EqualsLiteral(UNKNOWN_CONTENT_TYPE) &&
+        !contentType.IsEmpty()) {
       contentType = VIEWSOURCE_CONTENT_TYPE;
     }
 
@@ -595,7 +636,7 @@ nsViewSourceChannel::SetNotificationCallbacks(
 }
 
 NS_IMETHODIMP
-nsViewSourceChannel::GetSecurityInfo(nsISupports** aSecurityInfo) {
+nsViewSourceChannel::GetSecurityInfo(nsITransportSecurityInfo** aSecurityInfo) {
   NS_ENSURE_TRUE(mChannel, NS_ERROR_FAILURE);
 
   return mChannel->GetSecurityInfo(aSecurityInfo);
@@ -633,8 +674,7 @@ nsViewSourceChannel::GetBaseURI(nsIURI** aBaseURI) {
       return isc->GetBaseURI(aBaseURI);
     }
   }
-  *aBaseURI = mBaseURI;
-  NS_IF_ADDREF(*aBaseURI);
+  *aBaseURI = do_AddRef(mBaseURI).take();
   return NS_OK;
 }
 
@@ -728,22 +768,15 @@ nsViewSourceChannel::SetTopLevelContentWindowId(uint64_t aWindowId) {
 }
 
 NS_IMETHODIMP
-nsViewSourceChannel::GetTopBrowsingContextId(uint64_t* aId) {
+nsViewSourceChannel::GetBrowserId(uint64_t* aId) {
   return !mHttpChannel ? NS_ERROR_NULL_POINTER
-                       : mHttpChannel->GetTopBrowsingContextId(aId);
+                       : mHttpChannel->GetBrowserId(aId);
 }
 
 NS_IMETHODIMP
-nsViewSourceChannel::SetTopBrowsingContextId(uint64_t aId) {
+nsViewSourceChannel::SetBrowserId(uint64_t aId) {
   return !mHttpChannel ? NS_ERROR_NULL_POINTER
-                       : mHttpChannel->SetTopBrowsingContextId(aId);
-}
-
-NS_IMETHODIMP
-nsViewSourceChannel::GetFlashPluginState(
-    nsIHttpChannel::FlashPluginState* aResult) {
-  return !mHttpChannel ? NS_ERROR_NULL_POINTER
-                       : mHttpChannel->GetFlashPluginState(aResult);
+                       : mHttpChannel->SetBrowserId(aId);
 }
 
 NS_IMETHODIMP
@@ -829,18 +862,6 @@ nsViewSourceChannel::ShouldStripRequestBodyHeader(const nsACString& aMethod,
              : mHttpChannel->ShouldStripRequestBodyHeader(aMethod, aResult);
 }
 
-NS_IMETHODIMP
-nsViewSourceChannel::GetAllowPipelining(bool* aAllowPipelining) {
-  return !mHttpChannel ? NS_ERROR_NULL_POINTER
-                       : mHttpChannel->GetAllowPipelining(aAllowPipelining);
-}
-
-NS_IMETHODIMP
-nsViewSourceChannel::SetAllowPipelining(bool aAllowPipelining) {
-  return !mHttpChannel ? NS_ERROR_NULL_POINTER
-                       : mHttpChannel->SetAllowPipelining(aAllowPipelining);
-}
-
 NS_IMETHODIMP
 nsViewSourceChannel::GetAllowSTS(bool* aAllowSTS) {
   return !mHttpChannel ? NS_ERROR_NULL_POINTER
@@ -997,6 +1018,33 @@ nsViewSourceChannel::SetIsMainDocumentChannel(bool aValue) {
                        : mHttpChannel->SetIsMainDocumentChannel(aValue);
 }
 
+NS_IMETHODIMP nsViewSourceChannel::SetClassicScriptHintCharset(
+    const nsAString& aClassicScriptHintCharset) {
+  return !mHttpChannel ? NS_ERROR_NULL_POINTER
+                       : mHttpChannel->SetClassicScriptHintCharset(
+                             aClassicScriptHintCharset);
+}
+
+NS_IMETHODIMP nsViewSourceChannel::GetClassicScriptHintCharset(
+    nsAString& aClassicScriptHintCharset) {
+  return !mHttpChannel ? NS_ERROR_NULL_POINTER
+                       : mHttpChannel->GetClassicScriptHintCharset(
+                             aClassicScriptHintCharset);
+}
+
+NS_IMETHODIMP nsViewSourceChannel::SetDocumentCharacterSet(
+    const nsAString& aDocumentCharacterSet) {
+  return !mHttpChannel
+             ? NS_ERROR_NULL_POINTER
+             : mHttpChannel->SetDocumentCharacterSet(aDocumentCharacterSet);
+}
+
+NS_IMETHODIMP nsViewSourceChannel::GetDocumentCharacterSet(
+    nsAString& aDocumentCharacterSet) {
+  return !mHttpChannel
+             ? NS_ERROR_NULL_POINTER
+             : mHttpChannel->GetDocumentCharacterSet(aDocumentCharacterSet);
+}
 // Have to manually forward SetCorsPreflightParameters since it's [notxpcom]
 void nsViewSourceChannel::SetCorsPreflightParameters(
     const nsTArray<nsCString>& aUnsafeHeaders,
@@ -1015,13 +1063,14 @@ void nsViewSourceChannel::DisableAltDataCache() {
 
 NS_IMETHODIMP
 nsViewSourceChannel::LogBlockedCORSRequest(const nsAString& aMessage,
-                                           const nsACString& aCategory) {
+                                           const nsACString& aCategory,
+                                           bool aIsWarning) {
   if (!mHttpChannel) {
     NS_WARNING(
         "nsViewSourceChannel::LogBlockedCORSRequest mHttpChannel is null");
     return NS_ERROR_UNEXPECTED;
   }
-  return mHttpChannel->LogBlockedCORSRequest(aMessage, aCategory);
+  return mHttpChannel->LogBlockedCORSRequest(aMessage, aCategory, aIsWarning);
 }
 
 NS_IMETHODIMP
@@ -1036,6 +1085,11 @@ nsViewSourceChannel::LogMimeTypeMismatch(const nsACString& aMessageName,
                                            aContentType);
 }
 
+// FIXME: Should this forward to mHttpChannel? This was previously handled by a
+// default empty implementation.
+void nsViewSourceChannel::SetSource(
+    mozilla::UniquePtr<mozilla::ProfileChunkedBuffer> aSource) {}
+
 const nsTArray<mozilla::net::PreferredAlternativeDataTypeParams>&
 nsViewSourceChannel::PreferredAlternativeDataTypes() {
   if (mCacheInfoChannel) {
@@ -1044,24 +1098,17 @@ nsViewSourceChannel::PreferredAlternativeDataTypes() {
   return mEmptyArray;
 }
 
-void nsViewSourceChannel::SetIPv4Disabled() {
-  if (mHttpChannelInternal) {
-    mHttpChannelInternal->SetIPv4Disabled();
-  }
-}
-
-void nsViewSourceChannel::SetIPv6Disabled() {
-  if (mHttpChannelInternal) {
-    mHttpChannelInternal->SetIPv6Disabled();
-  }
-}
-
 void nsViewSourceChannel::DoDiagnosticAssertWhenOnStopNotCalledOnDestroy() {
   if (mHttpChannelInternal) {
     mHttpChannelInternal->DoDiagnosticAssertWhenOnStopNotCalledOnDestroy();
   }
 }
 
+// FIXME: Should this forward to mHttpChannelInternal? This was previously
+// handled by a default empty implementation.
+void nsViewSourceChannel::SetConnectionInfo(
+    mozilla::net::nsHttpConnectionInfo* aInfo) {}
+
 // nsIChildChannel methods
 
 NS_IMETHODIMP
@@ -1121,6 +1168,22 @@ nsViewSourceChannel::AsyncOnChannelRedirect(
   rv = newChannel->GetOriginalURI(getter_AddRefs(newChannelOrigURI));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsAutoCString scheme;
+  rv = newChannelOrigURI->GetScheme(scheme);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  nsCOMPtr<nsIIOService> ioService(do_GetIOService(&rv));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  rv = WillUseExternalProtocolHandler(ioService, scheme.get());
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
   nsCOMPtr<nsIURI> newChannelUpdatedOrigURI;
   rv = BuildViewSourceURI(newChannelOrigURI,
                           getter_AddRefs(newChannelUpdatedOrigURI));
