@@ -69,7 +69,7 @@ function pumpReadStream(inputStream, goon) {
       Ci.nsIInputStreamPump
     );
     pump.init(inputStream, 0, 0, true);
-    var data = "";
+    let data = "";
     pump.asyncRead({
       onStartRequest(aRequest) {},
       onDataAvailable(aRequest, aInputStream, aOffset, aCount) {
@@ -89,7 +89,7 @@ function pumpReadStream(inputStream, goon) {
     });
   } else {
     // blocking stream
-    var data = read_stream(inputStream, inputStream.available());
+    let data = read_stream(inputStream, inputStream.available());
     goon(data);
   }
 }
@@ -187,8 +187,8 @@ OpenCallback.prototype = {
         return;
       }
 
-      var self = this;
-      executeSoon(function() {
+      let self = this;
+      executeSoon(function () {
         // emulate network latency
         entry.setMetaDataElement("meto", self.workingMetadata);
         entry.metaDataReady();
@@ -205,12 +205,12 @@ OpenCallback.prototype = {
 
           return;
         }
-        executeSoon(function() {
+        executeSoon(function () {
           // emulate more network latency
           if (self.behavior & DOOMED) {
             LOG_C2(self, "checking doom state");
             try {
-              var os = entry.openOutputStream(0, -1);
+              let os = entry.openOutputStream(0, -1);
               // Unfortunately, in the undetermined state we cannot even check whether the entry
               // is actually doomed or not.
               os.close();
@@ -226,7 +226,7 @@ OpenCallback.prototype = {
 
           var offset = self.behavior & PARTIAL ? entry.dataSize : 0;
           LOG_C2(self, "openOutputStream @ " + offset);
-          var os = entry.openOutputStream(offset, -1);
+          let os = entry.openOutputStream(offset, -1);
           LOG_C2(self, "writing data");
           var wrt = os.write(self.workingData, self.workingData.length);
           Assert.equal(wrt, self.workingData.length);
@@ -249,8 +249,8 @@ OpenCallback.prototype = {
         this.goon(entry, true);
       }
 
-      var self = this;
-      pumpReadStream(entry.openInputStream(0), function(data) {
+      let self = this;
+      pumpReadStream(entry.openInputStream(0), function (data) {
         Assert.equal(data, self.workingData);
         self.onDataCheckPassed = true;
         LOG_C2(self, "entry read done");
@@ -269,7 +269,7 @@ OpenCallback.prototype = {
   throwAndNotify(entry) {
     LOG_C2(this, "Throwing");
     var self = this;
-    executeSoon(function() {
+    executeSoon(function () {
       LOG_C2(self, "Notifying");
       self.goon(entry);
     });
@@ -306,6 +306,7 @@ VisitCallback.prototype = {
     aURI,
     aIdEnhance,
     aDataSize,
+    aAltDataSize,
     aFetchCount,
     aLastModifiedTime,
     aExpirationTime,
@@ -387,7 +388,7 @@ MultipleCallbacks.prototype = {
     if (--this.pending == 0) {
       var self = this;
       if (this.delayed) {
-        executeSoon(function() {
+        executeSoon(function () {
           self.goon();
         });
       } else {
@@ -421,7 +422,7 @@ function wait_for_cache_index(continue_func) {
 }
 
 function finish_cache2_test() {
-  callbacks.forEach(function(callback, index) {
+  callbacks.forEach(function (callback, index) {
     callback.selfCheck();
   });
   do_test_finished();
