@@ -4,9 +4,7 @@
 
 "use strict";
 
-ChromeUtils.import("resource://testing-common/httpd.js");
-ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+const { HttpServer } = ChromeUtils.import("resource://testing-common/httpd.js");
 
 var httpserver = new HttpServer();
 httpserver.start(-1);
@@ -15,7 +13,7 @@ const PORT = httpserver.identity.primaryPort;
 function make_channel(url) {
   return NetUtil.newChannel({
     uri: url,
-    loadUsingSystemPrincipal: true
+    loadUsingSystemPrincipal: true,
   }).QueryInterface(Ci.nsIHttpChannel);
 }
 
@@ -27,10 +25,11 @@ function test_handler(metadata, response) {
   response.setHeader("Cache-Control", "no-cache");
   response.setHeader("ETag", "test-etag1");
 
+  let etag;
   try {
-    var etag = metadata.getHeader("If-None-Match");
-  } catch(ex) {
-    var etag = "";
+    etag = metadata.getHeader("If-None-Match");
+  } catch (ex) {
+    etag = "";
   }
 
   if (etag == "test-etag1") {
@@ -45,7 +44,7 @@ function test_handler(metadata, response) {
 
 function cached_handler(metadata, response) {
   response.setHeader("Content-Type", "text/plain");
-  response.setHeader("Cache-Control", "Cache-Control: max-age=3600");
+  response.setHeader("Cache-Control", "max-age=3600");
   response.setHeader("ETag", "test-etag1");
 
   response.setStatusLine(metadata.httpVersion, 200, "OK");
@@ -56,14 +55,20 @@ function cached_handler(metadata, response) {
 
 let gResponseCounter = 0;
 let gIsFromCache = 0;
-function checkContent(request, buffer, context, isFromCache)
-{
+function checkContent(request, buffer, context, isFromCache) {
   Assert.equal(buffer, gResponseBody);
+  info(
+    "isRacing: " +
+      request.QueryInterface(Ci.nsICacheInfoChannel).isRacing() +
+      "\n"
+  );
   gResponseCounter++;
   if (isFromCache) {
     gIsFromCache++;
   }
-  executeSoon(() => { testGenerator.next(); });
+  executeSoon(() => {
+    testGenerator.next();
+  });
 }
 
 function run_test() {
@@ -79,7 +84,7 @@ function run_test() {
 }
 
 let testGenerator = testSteps();
-function *testSteps() {
+function* testSteps() {
   /*
    * In this test, we have a relatively low timeout of 200ms and an assertion that
    * the timer works properly by checking that the time was greater than 200ms.
@@ -88,46 +93,61 @@ function *testSteps() {
    * 20ms.
    */
   Services.prefs.setBoolPref("privacy.reduceTimerPrecision", true);
-  Services.prefs.setIntPref("privacy.resistFingerprinting.reduceTimerPrecision.microseconds", 20000);
+  Services.prefs.setIntPref(
+    "privacy.resistFingerprinting.reduceTimerPrecision.microseconds",
+    20000
+  );
 
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     Services.prefs.clearUserPref("privacy.reduceTimerPrecision");
-    Services.prefs.clearUserPref("privacy.resistFingerprinting.reduceTimerPrecision.microseconds");
+    Services.prefs.clearUserPref(
+      "privacy.resistFingerprinting.reduceTimerPrecision.microseconds"
+    );
   });
 
   // Initial request. Stores the response in the cache.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  let channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 1);
   equal(g200Counter, 1, "check number of 200 responses");
   equal(g304Counter, 0, "check number of 304 responses");
 
   // Checks that response is returned from the cache, after a 304 response.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 2);
   equal(g200Counter, 1, "check number of 200 responses");
   equal(g304Counter, 1, "check number of 304 responses");
 
   // Checks that delaying the response from the cache works.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(200);
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(200);
   let startTime = Date.now();
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
-  greaterOrEqual(Date.now() - startTime, 200, "Check that timer works properly");
+  greaterOrEqual(
+    Date.now() - startTime,
+    200,
+    "Check that timer works properly"
+  );
   equal(gResponseCounter, 3);
   equal(g200Counter, 1, "check number of 200 responses");
   equal(g304Counter, 2, "check number of 304 responses");
 
   // Checks that we can trigger the cache open immediately, even if the cache delay is set very high.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
-  do_timeout(50, function() {
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerDelayedOpenCacheEntry();
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(100000);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
+  do_timeout(50, function () {
+    channel
+      .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+      .test_triggerDelayedOpenCacheEntry();
   });
   yield undefined;
   equal(gResponseCounter, 4);
@@ -135,11 +155,13 @@ function *testSteps() {
   equal(g304Counter, 3, "check number of 304 responses");
 
   // Sets a high delay for the cache fetch, and triggers the network activity.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
-  // Trigger network after 50 ms.
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(100000);
   channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(50);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
+  // Trigger network after 50 ms.
   yield undefined;
   equal(gResponseCounter, 5);
   equal(g200Counter, 2, "check number of 200 responses");
@@ -148,42 +170,43 @@ function *testSteps() {
   // Sets a high delay for the cache fetch, and triggers the network activity.
   // While the network response is produced, we trigger the cache fetch.
   // Because the network response was the first, a non-conditional request is sent.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
-  do_timeout(50, function() {
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(0);
-    executeSoon(() => { channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerDelayedOpenCacheEntry(); });
-  });
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(100000);
+  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(50);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 6);
   equal(g200Counter, 3, "check number of 200 responses");
   equal(g304Counter, 3, "check number of 304 responses");
 
   // Triggers cache open before triggering network.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
-  do_timeout(50, function() {
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerDelayedOpenCacheEntry();
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(0);
-  });
+  channel = make_channel("http://localhost:" + PORT + "/rcwn");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(100000);
+  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(5000);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_triggerDelayedOpenCacheEntry();
   yield undefined;
   equal(gResponseCounter, 7);
   equal(g200Counter, 3, "check number of 200 responses");
   equal(g304Counter, 4, "check number of 304 responses");
 
   // Load the cached handler so we don't need to revalidate
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 8);
   equal(g200Counter, 4, "check number of 200 responses");
   equal(g304Counter, 4, "check number of 304 responses");
 
   // Make sure response is loaded from the cache, not the network
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 9);
   equal(g200Counter, 4, "check number of 200 responses");
@@ -191,11 +214,13 @@ function *testSteps() {
 
   // Cache times out, so we trigger the network
   gIsFromCache = 0;
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
+  channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(100000);
   // trigger network after 50 ms
   channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(50);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 10);
   equal(gIsFromCache, 0, "should be from the network");
@@ -203,13 +228,12 @@ function *testSteps() {
   equal(g304Counter, 4, "check number of 304 responses");
 
   // Cache callback comes back right after network is triggered.
-  var channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
-  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-  channel.asyncOpen2(new ChannelListener(checkContent, null));
-  do_timeout(50, function() {
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(0);
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerDelayedOpenCacheEntry();
-  });
+  channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
+  channel
+    .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+    .test_delayCacheEntryOpeningBy(55);
+  channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(50);
+  channel.asyncOpen(new ChannelListener(checkContent, null));
   yield undefined;
   equal(gResponseCounter, 11);
   info("IsFromCache: " + gIsFromCache + "\n");
@@ -221,19 +245,14 @@ function *testSteps() {
   // and some we will get from the cache.
   gIsFromCache = 0;
   for (var i = 0; i < 50; i++) {
-    var channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
-    channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_delayCacheEntryOpeningBy(100000);
-    channel.asyncOpen2(new ChannelListener(checkContent, null));
+    channel = make_channel("http://localhost:" + PORT + "/rcwn_cached");
+    channel
+      .QueryInterface(Ci.nsIRaceCacheWithNetwork)
+      .test_delayCacheEntryOpeningBy(i * 100);
     channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerNetwork(10);
+    channel.asyncOpen(new ChannelListener(checkContent, null));
     // This may be racy. The delay was chosen because the distribution of net-cache
     // results was around 25-25 on my machine.
-    do_timeout(i*100, function() {
-      try {
-        channel.QueryInterface(Ci.nsIRaceCacheWithNetwork).test_triggerDelayedOpenCacheEntry();
-      } catch (e) {}
-
-    });
-
     yield undefined;
   }
 
