@@ -33,24 +33,24 @@ let redirectHook = "http-on-modify-request";
 var httpServer = null,
   httpServer2 = null;
 
-XPCOMUtils.defineLazyGetter(this, "port1", function() {
+XPCOMUtils.defineLazyGetter(this, "port1", function () {
   return httpServer.identity.primaryPort;
 });
 
-XPCOMUtils.defineLazyGetter(this, "port2", function() {
+XPCOMUtils.defineLazyGetter(this, "port2", function () {
   return httpServer2.identity.primaryPort;
 });
 
 // Test Part 1: a cross-path redirect on a single HTTP server
 // http://localhost:port1/bait -> http://localhost:port1/switch
 var baitPath = "/bait";
-XPCOMUtils.defineLazyGetter(this, "baitURI", function() {
+XPCOMUtils.defineLazyGetter(this, "baitURI", function () {
   return "http://localhost:" + port1 + baitPath;
 });
 var baitText = "you got the worm";
 
 var redirectedPath = "/switch";
-XPCOMUtils.defineLazyGetter(this, "redirectedURI", function() {
+XPCOMUtils.defineLazyGetter(this, "redirectedURI", function () {
   return "http://localhost:" + port1 + redirectedPath;
 });
 var redirectedText = "worms are not tasty";
@@ -58,25 +58,25 @@ var redirectedText = "worms are not tasty";
 // Test Part 2: Now, a redirect to a different server
 // http://localhost:port1/bait2 -> http://localhost:port2/switch
 var bait2Path = "/bait2";
-XPCOMUtils.defineLazyGetter(this, "bait2URI", function() {
+XPCOMUtils.defineLazyGetter(this, "bait2URI", function () {
   return "http://localhost:" + port1 + bait2Path;
 });
 
-XPCOMUtils.defineLazyGetter(this, "redirected2URI", function() {
+XPCOMUtils.defineLazyGetter(this, "redirected2URI", function () {
   return "http://localhost:" + port2 + redirectedPath;
 });
 
 // Test Part 3, begin with a serverside redirect that itself turns into an instance
 // of Test Part 1
 var bait3Path = "/bait3";
-XPCOMUtils.defineLazyGetter(this, "bait3URI", function() {
+XPCOMUtils.defineLazyGetter(this, "bait3URI", function () {
   return "http://localhost:" + port1 + bait3Path;
 });
 
 // Test Part 4, begin with this client-side redirect and which then redirects
 // to an instance of Test Part 1
 var bait4Path = "/bait4";
-XPCOMUtils.defineLazyGetter(this, "bait4URI", function() {
+XPCOMUtils.defineLazyGetter(this, "bait4URI", function () {
   return "http://localhost:" + port1 + bait4Path;
 });
 
@@ -162,7 +162,7 @@ function makeAsyncTest(uri, headerValue, nextTask) {
 
   // Produce a callback function which checks for the presence of headerValue,
   // and then continues to the next async test task
-  var verifier = function(req, buffer) {
+  var verifier = function (req, buffer) {
     if (!(req instanceof Ci.nsIHttpChannel)) {
       do_throw(req + " is not an nsIHttpChannel, catastrophe imminent!");
     }
@@ -174,7 +174,7 @@ function makeAsyncTest(uri, headerValue, nextTask) {
   };
 
   // Produce a function to run an asyncOpen test using the above verifier
-  var test = function() {
+  var test = function () {
     var chan = make_channel(uri);
     chan.asyncOpen(new ChannelListener(verifier));
   };
@@ -206,12 +206,13 @@ function runXHRTest(uri, headerValue) {
 }
 
 function done() {
-  httpServer.stop(function() {
+  httpServer.stop(function () {
     httpServer2.stop(do_test_finished);
   });
 }
 
-var redirector = new Redirector();
+// Needed for side-effects
+new Redirector();
 
 function run_test() {
   httpServer = new HttpServer();
