@@ -112,7 +112,7 @@ function test_echo() {
       let serverOutput = connection.openOutputStream(0, 0, 0);
 
       serverAsyncInput.asyncWait(
-        function(aStream) {
+        function (aStream) {
           info("called test_echo's server's onInputStreamReady");
           let serverScriptableInput = new ScriptableInputStream(aStream);
 
@@ -157,7 +157,7 @@ function test_echo() {
   info("client has written to server");
 
   clientAsyncInput.asyncWait(
-    function(aStream) {
+    function (aStream) {
       info("called test_echo's client's onInputStreamReady");
       log += "c";
 
@@ -235,7 +235,7 @@ function test_no_such_socket() {
     .openInputStream(0, 0, 0)
     .QueryInterface(Ci.nsIAsyncInputStream);
   clientAsyncInput.asyncWait(
-    function(aStream) {
+    function (aStream) {
       info("called test_no_such_socket's onInputStreamReady");
 
       Assert.equal(aStream, clientAsyncInput);
@@ -381,7 +381,7 @@ function test_connect_permission() {
     .openInputStream(0, 0, 0)
     .QueryInterface(Ci.nsIAsyncInputStream);
   client1AsyncInput.asyncWait(
-    function(aStream) {
+    function (aStream) {
       info("called test_connect_permission's client1's onInputStreamReady");
       log += "1";
 
@@ -399,14 +399,13 @@ function test_connect_permission() {
       dirName.permissions = allPermissions;
       socketName.permissions = 0;
 
-      let client2 = socketTransportService.createUnixDomainTransport(
-        socketName
-      );
+      let client2 =
+        socketTransportService.createUnixDomainTransport(socketName);
       let client2AsyncInput = client2
         .openInputStream(0, 0, 0)
         .QueryInterface(Ci.nsIAsyncInputStream);
       client2AsyncInput.asyncWait(
-        function(aStream) {
+        function (aStream) {
           info("called test_connect_permission's client2's onInputStreamReady");
           log += "2";
 
@@ -421,9 +420,8 @@ function test_connect_permission() {
           // Now make everything accessible, and try one last time.
           socketName.permissions = allPermissions;
 
-          client3 = socketTransportService.createUnixDomainTransport(
-            socketName
-          );
+          client3 =
+            socketTransportService.createUnixDomainTransport(socketName);
 
           let client3Output = client3.openOutputStream(0, 0, 0);
           client3Output.write("Hanratty", 8);
@@ -458,7 +456,7 @@ function test_connect_permission() {
     let serverOutput = aTransport.openOutputStream(0, 0, 0);
 
     serverInput.asyncWait(
-      function(aStream) {
+      function (aStream) {
         info(
           "called test_connect_permission's socketAccepted's onInputStreamReady"
         );
@@ -670,9 +668,8 @@ function test_abstract_address_socket() {
     onStopListening: (aServ, aTransport) => {},
   });
 
-  let client = socketTransportService.createUnixDomainAbstractAddressTransport(
-    socketname
-  );
+  let client =
+    socketTransportService.createUnixDomainAbstractAddressTransport(socketname);
   Assert.equal(client.host, socketname);
   Assert.equal(client.port, 0);
   let clientInput = client
